from django.test import TestCase

from .. import factories


class SecurityGroupHandlerTest(TestCase):

    def test_cloud_service_name_is_copied_from_provider_settings_on_spl_creation(self):
        settings = factories.AzureServiceSettingsFactory()
        settings.options = {'cloud_service_name': 'azure-cloud'}
        settings.save()
        azure_service = factories.AzureServiceFactory(settings=settings)

        spl = factories.AzureServiceProjectLinkFactory(service=azure_service)

        self.assertEquals(spl.cloud_service_name, settings.options['cloud_service_name'])

    def test_cloud_service_name_is_not_populated_if_it_is_empty_in_settings(self):
        spl = factories.AzureServiceProjectLinkFactory()

        self.assertEqual('', spl.cloud_service_name)

    def test_cloud_service_name_is_not_overwritten_on_creation_if_it_is_passed_explicitely(self):
        settings = factories.AzureServiceSettingsFactory()
        spl_cloud_name = 'spl-cloud'
        settings.options = {'cloud_service_name': 'azure-cloud'}
        settings.save()
        azure_service = factories.AzureServiceFactory(settings=settings)

        spl = factories.AzureServiceProjectLinkFactory(service=azure_service, cloud_service_name=spl_cloud_name)

        self.assertEquals(spl.cloud_service_name, spl_cloud_name)



import mock
import httplib
import unittest

from libcloud.common.types import LibcloudError

from ..driver import AzureResponse


class ErrorMessageParsingTest(unittest.TestCase):
    def setUp(self):
        self._mock_response = mock.Mock()
        self._mock_response.getheaders.return_value = []
        self._mock_response.status = httplib.OK
        self._mock_response._original_data = None
        self._mock_connection = mock.Mock()

    def test_azure_response_parses_error_message(self):
        self._mock_response.status = httplib.NOT_FOUND
        self._mock_response.read.return_value = '<Error xmlns="http://schemas.microsoft.com/windowsazure" xmlns:i="http://www.w3.org/2001/XMLSchema-instance"><Code>ResourceNotFound</Code><Message>The resource service name hostedservices is not supported.</Message></Error>'

        # response object has been updated in 2.1
        self._mock_response.status_code = self._mock_response.status
        self._mock_response.headers = []
        self._mock_response.text = self._mock_response.read.return_value
        response = AzureResponse(response=self._mock_response,
                                 connection=self._mock_connection)
        try:
            response.parse_error()
        except LibcloudError as e:
            self.assertEqual(e.value, 'ResourceNotFound: The resource service name hostedservices is not supported. Status code: 404.')
        else:
            self.fail('Exception is not thrown')



from django.utils.functional import cached_property

from nodeconductor.structure.tests.fixtures import ProjectFixture

from . import factories


class AzureFixture(ProjectFixture):

    @cached_property
    def service(self):
        return factories.AzureServiceFactory(customer=self.customer)

    @cached_property
    def spl(self):
        return factories.AzureServiceProjectLinkFactory(service=self.service, project=self.project)

    @cached_property
    def virtual_machine(self):
        return factories.VirtualMachineFactory(service_project_link=self.spl)



from ddt import data, ddt
from rest_framework import test, status
from libcloud.compute.types import NodeState
import mock

from . import fixtures, factories
from .. import models


@ddt
class VirtualMachineRDPTest(test.APITransactionTestCase):

    def setUp(self):
        self.fixture = fixtures.AzureFixture()

    def test_rdp_returns_xrdp_file_as_attachment(self):
        self.client.force_authenticate(self.fixture.owner)
        vm = self.fixture.virtual_machine
        factories.InstanceEndpoint(instance=vm, name=models.InstanceEndpoint.Name.RDP)
        url = factories.VirtualMachineFactory.get_url(vm, 'rdp')

        response = self.client.get(url)

        self.assertEquals(response.status_code, status.HTTP_200_OK)
        self.assertIn('%s.rdp' % vm.name, response._headers['content-disposition'][1])

    def test_rdp_is_not_available_if_backend_vm_does_not_have_a_port(self):
        self.client.force_authenticate(self.fixture.owner)
        vm = self.fixture.virtual_machine
        url = factories.VirtualMachineFactory.get_url(vm, 'rdp')

        response = self.client.get(url)

        self.assertEquals(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_rdp_action_is_not_available_only_to_erred_or_instances(self):
        self.client.force_authenticate(self.fixture.owner)
        vm = self.fixture.virtual_machine
        vm.state = vm.States.ERRED
        vm.save()
        url = factories.VirtualMachineFactory.get_url(vm, 'rdp')

        response = self.client.get(url)

        self.assertEquals(response.status_code, status.HTTP_409_CONFLICT)

    @data(NodeState.STOPPED, NodeState.UNKNOWN, NodeState.ERROR, NodeState.STARTING)
    def test_rdp_action_is_allowed_only_for_running_instances(self, runtime_state):
        self.client.force_authenticate(self.fixture.owner)
        vm = self.fixture.virtual_machine
        vm.runtime_state = runtime_state
        vm.save()
        url = factories.VirtualMachineFactory.get_url(vm, 'rdp')

        response = self.client.get(url)

        self.assertEquals(response.status_code, status.HTTP_409_CONFLICT)


@ddt
class VirtualMachineStartTest(test.APITransactionTestCase):

    def setUp(self):
        self.fixture = fixtures.AzureFixture()
        self.client.force_authenticate(self.fixture.owner)

    @mock.patch('nodeconductor_azure.executors.VirtualMachineStartExecutor.execute')
    def test_stopped_machine_can_be_started(self, start_executor_mock):
        vm = self.fixture.virtual_machine
        vm.runtime_state = NodeState.STOPPED
        vm.save()
        url = factories.VirtualMachineFactory.get_url(vm, 'start')

        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_202_ACCEPTED)

    @mock.patch('nodeconductor_azure.executors.VirtualMachineStartExecutor.execute')
    def test_machine_in_ok_state_can_be_started(self, start_executor_mock):
        vm = self.fixture.virtual_machine
        vm.state = vm.States.OK
        vm.runtime_state = NodeState.STOPPED
        vm.save()
        url = factories.VirtualMachineFactory.get_url(vm, 'start')

        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_202_ACCEPTED)

    @data(NodeState.RUNNING, NodeState.UNKNOWN, NodeState.ERROR, NodeState.STARTING)
    @mock.patch('nodeconductor_azure.executors.VirtualMachineStartExecutor.execute')
    def test_only_stopped_machine_can_be_started(self, runtime_state, start_executor_mock):
        vm = self.fixture.virtual_machine
        vm.runtime_state = runtime_state
        vm.save()
        url = factories.VirtualMachineFactory.get_url(vm, 'start')

        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_409_CONFLICT)

    @data(models.VirtualMachine.States.DELETING, models.VirtualMachine.States.UPDATING,
          models.VirtualMachine.States.DELETION_SCHEDULED, models.VirtualMachine.States.ERRED)
    @mock.patch('nodeconductor_azure.executors.VirtualMachineStartExecutor.execute')
    def test_only_machine_in_ok_state_can_be_started(self, state, start_executor_mock):
        vm = self.fixture.virtual_machine
        vm.state = state
        vm.save()
        url = factories.VirtualMachineFactory.get_url(vm, 'start')

        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_409_CONFLICT)


@ddt
class VirtualMachineStopTest(test.APITransactionTestCase):

    def setUp(self):
        self.fixture = fixtures.AzureFixture()
        self.client.force_authenticate(self.fixture.owner)

    @mock.patch('nodeconductor_azure.executors.VirtualMachineStopExecutor.execute')
    def test_running_machine_can_be_stopped(self, stop_executor_mock):
        vm = self.fixture.virtual_machine
        vm.runtime_state = NodeState.RUNNING
        vm.save()
        url = factories.VirtualMachineFactory.get_url(vm, 'stop')

        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_202_ACCEPTED)

    @mock.patch('nodeconductor_azure.executors.VirtualMachineStopExecutor.execute')
    def test_machine_in_ok_state_can_be_stopped(self, stop_executor_mock):
        vm = self.fixture.virtual_machine
        vm.state = vm.States.OK
        vm.save()
        url = factories.VirtualMachineFactory.get_url(vm, 'stop')

        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_202_ACCEPTED)

    @data(NodeState.STOPPED, NodeState.UNKNOWN, NodeState.ERROR, NodeState.STARTING)
    @mock.patch('nodeconductor_azure.executors.VirtualMachineStopExecutor.execute')
    def test_only_running_machine_can_be_stopped(self, runtime_state, stop_executor_mock):
        vm = self.fixture.virtual_machine
        vm.runtime_state = runtime_state
        vm.save()
        url = factories.VirtualMachineFactory.get_url(vm, 'stop')

        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_409_CONFLICT)

    @data(models.VirtualMachine.States.DELETING, models.VirtualMachine.States.UPDATING,
          models.VirtualMachine.States.DELETION_SCHEDULED, models.VirtualMachine.States.ERRED)
    @mock.patch('nodeconductor_azure.executors.VirtualMachineStopExecutor.execute')
    def test_only_machine_in_ok_state_can_be_stopped(self, state, stop_executor_mock):
        vm = self.fixture.virtual_machine
        vm.state = state
        vm.save()
        url = factories.VirtualMachineFactory.get_url(vm, 'stop')

        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_409_CONFLICT)



from nodeconductor.core import tasks as core_tasks

from backend import AzureBackendError


class StateException(Exception):
    pass


class PollCheckTask(core_tasks.Task):
    # TODO [TM:8/14/17] merge with OpenStack check task
    max_retries = 300
    default_retry_delay = 5

    @classmethod
    def get_description(cls, virtual_machine, backend_check_method, *args, **kwargs):
        return 'Check virtual machine "%s" with method "%s"' % (virtual_machine, backend_check_method)

    def get_backend(self, virtual_machine):
        return virtual_machine.get_backend()

    def execute(self, virtual_machine, backend_check_method):
        # backend_check_method should return True if object does not exist at backend
        backend = self.get_backend(virtual_machine)
        if not getattr(backend, backend_check_method)(virtual_machine):
            self.retry()
        return virtual_machine


class PollRuntimeStateTask(core_tasks.Task):
    max_retries = 300
    default_retry_delay = 5

    @classmethod
    def get_description(cls, virtual_machine, backend_pull_method, *args, **kwargs):
        return 'Poll virtual machine "%s" with method "%s"' % (virtual_machine, backend_pull_method)

    def get_backend(self, virtual_machine):
        return virtual_machine.get_backend()

    def execute(self, virtual_machine, backend_pull_method, success_state, erred_state):
        backend = self.get_backend(virtual_machine)
        try:
            getattr(backend, backend_pull_method)(virtual_machine)
        except AzureBackendError:
            self.retry()

        virtual_machine.refresh_from_db()
        if virtual_machine.runtime_state not in (backend.State.fromstring(success_state),
                                                 backend.State.fromstring(erred_state)):
            self.retry()
        elif virtual_machine.runtime_state == backend.State.fromstring(erred_state):
            raise StateException('%s %s (PK: %s) state become erred: %s' % (
                    virtual_machine.__class__.__name__, virtual_machine, virtual_machine.pk, erred_state))
        return virtual_machine



from django.urls import reverse
from libcloud.compute.types import NodeState

import factory

from nodeconductor.structure.tests import factories as structure_factories
from nodeconductor.structure import models as structure_models

from .. import models


class AzureServiceSettingsFactory(structure_factories.ServiceSettingsFactory):
    class Meta(object):
        model = structure_models.ServiceSettings

    type = 'Azure'
    customer = factory.SubFactory(structure_factories.CustomerFactory)


class AzureServiceFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.AzureService

    settings = factory.SubFactory(AzureServiceSettingsFactory)
    customer = factory.SelfAttribute('settings.customer')

    @classmethod
    def get_url(cls, service=None, action=None):
        if service is None:
            service = AzureServiceFactory()
        url = 'http://testserver' + reverse('azure-detail', kwargs={'uuid': service.uuid})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('azure-list')


class AzureServiceProjectLinkFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.AzureServiceProjectLink

    service = factory.SubFactory(AzureServiceFactory)
    project = factory.SubFactory(structure_factories.ProjectFactory)

    @classmethod
    def get_url(cls, spl=None, action=None):
        if spl is None:
            spl = AzureServiceProjectLinkFactory()
        url = 'http://testserver' + reverse('azure-spl-detail', kwargs={'pk': spl.pk})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('azure-spl-list')


class VirtualMachineFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.VirtualMachine

    name = factory.Sequence(lambda n: 'virtual-machine%s' % n)
    backend_id = factory.Sequence(lambda n: 'virtual-machine-id%s' % n)
    service_project_link = factory.SubFactory(AzureServiceProjectLinkFactory)

    state = models.VirtualMachine.States.OK
    runtime_state = NodeState.RUNNING
    cores = factory.fuzzy.FuzzyInteger(1, 8, step=2)
    ram = factory.fuzzy.FuzzyInteger(1024, 10240, step=1024)
    disk = factory.fuzzy.FuzzyInteger(1024, 102400, step=1024)

    @classmethod
    def get_url(cls, instance=None, action=None):
        if instance is None:
            instance = VirtualMachineFactory()
        url = 'http://testserver' + reverse('azure-virtualmachine-detail', kwargs={'uuid': instance.uuid})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('aws-virtualmachine-list')


class InstanceEndpoint(factory.DjangoModelFactory):
    class Meta(object):
        model = models.InstanceEndpoint

    name = factory.fuzzy.FuzzyChoice(choices=models.InstanceEndpoint.Name.CHOICES)
    protocol = factory.fuzzy.FuzzyChoice(choices=models.InstanceEndpoint.Protocol.CHOICES)
    local_port = factory.fuzzy.FuzzyInteger(1000, 65535)
    public_port = factory.fuzzy.FuzzyInteger(1000, 65535)
    instance = factory.SubFactory(VirtualMachineFactory)
